@@ -8,7 +8,7 @@
 //
 // dVect.cpp: Classess and functions for vectorization
 //
-// Copyright (C) 2011	Xiuwen Zheng
+// Copyright (C) 2012	Xiuwen Zheng
 //
 // This file is part of CoreArray.
 //
@@ -25,6 +25,7 @@
 // License along with CoreArray.
 // If not, see <http://www.gnu.org/licenses/>.
 
+
 #include <dVect.h>
 
 #ifdef COREARRAY_VT_SIMD
@@ -41,10 +42,10 @@
 
 // SSE intrinsics
 
-inline static float _add(const float v1, const float v2) { return v1+v2; }
-inline static float _sub(const float v1, const float v2) { return v1-v2; }
-inline static float _mul(const float v1, const float v2) { return v1*v2; }
-inline static float _div(const float v1, const float v2) { return v1/v2; }
+COREARRAY_FORCE_INLINE static float _add(const float v1, const float v2) { return v1+v2; }
+COREARRAY_FORCE_INLINE static float _sub(const float v1, const float v2) { return v1-v2; }
+COREARRAY_FORCE_INLINE static float _mul(const float v1, const float v2) { return v1*v2; }
+COREARRAY_FORCE_INLINE static float _div(const float v1, const float v2) { return v1/v2; }
 
 // Unroll loop
 
@@ -389,10 +390,10 @@ float CORESSECALL CoreArray::Vectorization::_SSE_DotProd_16(const float *x, cons
 
 // SSE2 intrinsics
 
-inline static double _add(const double v1, const double v2) { return v1+v2; }
-inline static double _sub(const double v1, const double v2) { return v1-v2; }
-inline static double _mul(const double v1, const double v2) { return v1*v2; }
-inline static double _div(const double v1, const double v2) { return v1/v2; }
+COREARRAY_FORCE_INLINE static double _add(const double v1, const double v2) { return v1+v2; }
+COREARRAY_FORCE_INLINE static double _sub(const double v1, const double v2) { return v1-v2; }
+COREARRAY_FORCE_INLINE static double _mul(const double v1, const double v2) { return v1*v2; }
+COREARRAY_FORCE_INLINE static double _div(const double v1, const double v2) { return v1/v2; }
 
 #define SSE2_L3_F64(Fd, Fs1, Fs2, Op, OpR) \
 	{ \
